*/
    QuickMSG
    Copyright (C) 2014  Jeroen Vreeken <jeroen@vreeken.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "attachment.h"

#include <string.h>

struct attachment *attachment_alloc(void)
{
	struct attachment *a;
	
	a = calloc(1, sizeof(struct attachment));
	
	return a;
}

void attachment_free(struct attachment *a)
{
	free(a->type);
	free(a->basetype);
	free(a->data);
	free(a);
}

int attachment_data_set(struct attachment *a, const void *data, const size_t length)
{
	if (a->data)
		free(a->data);
	
	a->data = malloc(length + 1);
	if (!a->data)
		return -1;
	
	memcpy(a->data, data, length);
	a->length = length;
	((char *)a->data)[a->length] = 0;
	
	return 0;
}

int attachment_type_set(struct attachment *a, const char *type)
{
	if (a->type)
		free(a->type);
	
	a->type = strdup(type);
	if (!a->type)
		return -1;
	
	return 0;
}

int attachment_basetype_set(struct attachment *a, const char *basetype)
{
	if (a->basetype)
		free(a->basetype);
	
	a->basetype = strdup(basetype);
	if (!a->basetype)
		return -1;
	
	return 0;
}

struct attachment_list *attachment_list_alloc(void)
{
	struct attachment_list *l;
	
	l = calloc(1, sizeof(struct attachment_list));
	
	return l;
}

void attachment_list_free(struct attachment_list *l)
{
	size_t i;
	
	for (i = 0; i < l->size; i++) {
		free(l->attachments[i]);
	}
	
	free(l->attachments);
	free(l);
}

int attachment_list_add(struct attachment_list *l, struct attachment *a)
{
	struct attachment **as;
	size_t s = l->size + 1;
	
	as = realloc(l->attachments, sizeof(struct attachment *) * s);
	if (!as)
		return -1;
		
	l->attachments = as;
	l->attachments[l->size] = a;
	l->size = s;
	
	return 0;
}

struct attachment *attachment_list_remove(struct attachment_list *l, size_t c)
{
	struct attachment *co = l->attachments[c];
	
	memmove(&l->attachments[c], &l->attachments[c+1], 
	    (l->size - c) * sizeof(struct attachment *));
	
	l->size--;
	
	return co;
}
