*/
    QuickMSG
    Copyright (C) 2014  Jeroen Vreeken <jeroen@vreeken.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#define _GNU_SOURCE
#include "post.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

struct post {
	int id;
	int from;
	time_t time;
	char *text;
	char *uri;
};

struct post *post_alloc(void)
{
	return calloc(1, sizeof(struct post));
}

void post_free(struct post *post)
{
	if (post) {
		free(post->text);
		free(post->uri);
	}
	free(post);
}

int post_id_set(struct post *post, int id)
{
	post->id = id;
	return 0;
}

int post_id_get(struct post *post)
{
	return post->id;
}

int post_from_set(struct post *post, int from)
{
	post->from = from;
	return 0;
}


int post_from_get(struct post *post)
{
	return post->from;
}

int post_time_set(struct post *post, time_t t)
{
	post->time = t;
	return 0;
}

time_t post_time_get(struct post *post)
{
	return post->time;
}

int post_text_set(struct post *post, char *text)
{
	post->text = strdup(text);
	return 0;
}

char *post_text_get(struct post *post)
{
	return post->text;
}

int post_uri_set(struct post *post, char *uri)
{
	post->uri = strdup(uri);
	return 0;
}

char *post_uri_get(struct post *post)
{
	return post->uri;
}


struct post_list *post_list_alloc(void)
{
	return calloc(1, sizeof(struct post_list));
}

void post_list_free(struct post_list *l)
{
	size_t i;
	
	for (i = 0; i < l->size; i++) {
		free(l->posts[i]);
	}
	
	free(l->posts);
	free(l);
}


int post_list_add(struct post_list *l, struct post *c)
{
	struct post **cs;
	size_t s = l->size + 1;
	
	cs = realloc(l->posts, sizeof(struct post *) * s);
	if (!cs)
		return -1;
	
	l->posts = cs;
	l->posts[l->size] = c;
	l->size = s;
	
	return 0;
}

struct post *post_list_remove(struct post_list *l, size_t c)
{
	struct post *co = l->posts[c];
	
	memmove(&l->posts[c], &l->posts[c+1], 
	    (l->size - c) * sizeof(struct post *));
	
	l->size--;
	
	return co;
}

char *post_print_header(void)
{
	return "id\tfrom\ttime\turi\ntext\n\n";
}

char *post_print(struct post *p)
{
	char *s;
	
	asprintf(&s, "%d\t%d\t%ld\t%s\n%s\n\n",
	    p->id, p->from, (long)p->time,
	    p->uri, p->text);
	
	return s;
}
